/*
								+----------------------------------+
								|                                  |
								| ***   Multi-platform timer   *** |
								|                                  |
								|   Copyright  -tHE SWINe- 2006   |
								|                                  |
								|             Timer.h              |
								|                                  |
								+----------------------------------+
*/

/*
 *	passed code revision at 08-23-2006
 *
 *	removed some unnecessary #defines for linux
 *	added higher precision timer for linux (instead of clock(), gettimeofday() can be used)
 *
 */

#if defined(WIN32)
#include <windows.h>
#endif
#include "Timer.h"

#if defined(WIN32)

CTimer::CTimer()
{
	QueryPerformanceFrequency(&m_n_freq);
	m_n_time = -1;
}

void CTimer::ResetTimer()
{
	m_f_time = 0;
	LARGE_INTEGER n_cur_time;
	QueryPerformanceCounter(&n_cur_time);
	m_n_time = n_cur_time.QuadPart;
}

float CTimer::f_Time()
{
	LARGE_INTEGER n_cur_time;

	QueryPerformanceCounter(&n_cur_time);
	if(m_n_time == -1)
		m_f_time = 0;
	else
		m_f_time += (float)((signed)(n_cur_time.QuadPart - m_n_time)) / m_n_freq.QuadPart;
	m_n_time = n_cur_time.QuadPart;

	return m_f_time;
}

#elif defined(TIMER_ALLOW_GETTIMEOFDAY)

#include <unistd.h>
#include <sys/time.h>

CTimer::CTimer()
{
	m_n_freq = 1000000; // microseconds
	m_n_time = (unsigned int)-1;
}

void CTimer::ResetTimer()
{
	m_f_time = 0;

	timeval t_tmp_time;
    gettimeofday(&t_tmp_time, NULL);
	m_n_time = t_tmp_time.tv_sec * 1000000 + t_tmp_time.tv_usec;
}

float CTimer::f_Time()
{
	timeval t_tmp_time;
    gettimeofday(&t_tmp_time, NULL);
	if(m_n_time == (unsigned int)-1)
		m_f_time = 0;
	else {
		m_f_time += (float)((signed)((t_tmp_time.tv_sec * 1000000 +
			t_tmp_time.tv_usec) - m_n_time)) * .000001f;
	}
	m_n_time = t_tmp_time.tv_sec * 1000000 + t_tmp_time.tv_usec;

	return m_f_time;
}

#else

#include <time.h>

CTimer::CTimer()
{
	m_n_freq = CLOCKS_PER_SEC;
	m_n_time = (unsigned int)-1;
}

void CTimer::ResetTimer()
{
	m_f_time = 0;
	m_n_time = clock();
}

float CTimer::f_Time()
{
	unsigned long n_cur_time;

	n_cur_time = clock();
	if(m_n_time == (unsigned int)-1)
		m_f_time = 0;
	else
		m_f_time += (float)((signed)(n_cur_time - m_n_time)) / m_n_freq;
	m_n_time = n_cur_time;

	return m_f_time;
}

#endif
