/*
								+----------------------------------+
								|                                  |
								|       ***  Vector math  ***      |
								|                                  |
								|   Copyright  -tHE SWINe- 2005   |
								|                                  |
								|             Vector.h             |
								|                                  |
								+----------------------------------+
*/

/*
 *	passed code revision at 05-16-2006
 *
 *	optimized some parts of code
 *	all integers storing just true of false were replaced with bool
 *
 *	passed code revision at 07-02-2006
 *
 *	added swizzle operators to vector classes
 *	added redlect / refract / fresnel functions to Vector3<T> class
 *	renamed Vector3<T>::Len to Vector3<T>::f_Length, Vector3<T>::Len2 to Vector3<T>::f_Length2
 *	added some Matrix3f methods, renamed Matrix3f::Init() to Matrix3f::Identity()
 *	renamed Planef to Plane3f
 *	renamed Pi to f_pi
 *	general cleanup of code
 *
 *	07-17-2006
 *
 *	renamed CPolygon::r_Vertex(int) to CPolygon::r_t_Vertex(int)
 *	renamed CPolygon::r_Normal() to CPolygon::r_t_Normal()
 *
 *	07-20-2006
 *
 *	added CPolygon::b_MT_RayHit_CullFrontfaces and CPolygon::b_MT_RayHit_CullBackFaces
 *
 *	07-24-2006
 *
 *	renamed TVertexStruct to TVertex3f
 *	added class CVectorMath with ray distances and bounding box / line (ray) intersection tests
 *
 *	01-23-2007
 *
 *	fixed std::vector handling in template CPolygon<TVertex3f> (not-enough-memory checks)
 *	remade TVertex3f base class and CPolygon<TVertex3f> a bit (added + and * operators,
 *	renamed Lerp() to t_Lerp())
 *
 *	note this was modified for convex madness series and most of stuff was teared down
 *
 */

#ifndef __VECTOR2_INCLUDED
#define __VECTOR2_INCLUDED

#if defined(_MSC_VER) && !defined(__MWERKS__) && !defined(for)
#define for if(0) {} else for
#endif

extern const float f_pi; // 3.1415926535897932384626433832795028841971691075f
extern const float f_epsilon; // 0.01f

#define swizzle2_in(a,b,ab) inline void ab(Vector2<T> &r_v_vec) \
	{ \
		a = r_v_vec.a; \
		b = r_v_vec.b; \
	}
#define swizzle3_in(a,b,c,abc) inline void abc(Vector3<T> &r_v_vec) \
	{ \
		a = r_v_vec.a; \
		b = r_v_vec.b; \
		c = r_v_vec.c; \
	}
#define swizzle4_in(a,b,c,d,abcd) inline void abcd(Vector4<T> &r_v_vec) \
	{ \
		a = r_v_vec.a; \
		b = r_v_vec.b; \
		c = r_v_vec.c; \
		d = r_v_vec.d; \
	}
#define swizzle2_out(a,b,v_ab) inline Vector2<T> v_ab() const \
	{ \
		return Vector2<T>(a, b); \
	}
#define swizzle3_out(a,b,c,v_abc) inline Vector3<T> v_abc() const \
	{ \
		return Vector3<T>(a, b, c); \
	}
#define swizzle4_out(a,b,c,d,v_abcd) inline Vector4<T> v_abcd() const \
	{ \
		return Vector4<T>(a, b, c, d); \
	}

/*
 *								=== Vector2 ===
 */

template <class T> struct Vector2 {
	T x, y;
	//
	inline Vector2() { }
	inline Vector2(T t_x, T t_y) :x(t_x), y(t_y) { }
	//
	inline T f_Length() const { return (T)sqrt(x * x + y * y); }
	inline T f_Length2() const { return x * x + y * y; }
	//
	inline T operator [](int n_index) const
	{
		if(&x + 1 == &y) // should optimize away in release mode
			return *(&x + n_index);
		else {
			if(n_index == 0)
				return x;
			else
				return y;
		}
	}

	inline T &operator [](int n_index)
	{
		if(&x + 1 == &y) // should optimize away in release mode
			return *(&x + n_index);
		else {
			if(n_index == 0)
				return x;
			else
				return y;
		}
	}

	inline void Normalize()
	{
		T t = f_Length();
		
		if(t != 0) {
			x /= t;
			y /= t;
		}
	}

	inline void Normalize(T t_len)
	{
		T t = f_Length();

		if(t != 0) {
			t /= t_len;
			x /= t;
			y /= t;
		}
	}

	inline bool operator ==(const Vector2<T> &r_v_vec) const
	{
		return x == r_v_vec.x && y == r_v_vec.y;
	}

	inline Vector2<T> operator +(const Vector2<T> &r_v_vec) const
	{
		return Vector2<T>(x + r_v_vec.x, y + r_v_vec.y);
	}

	inline Vector2<T> operator -(const Vector2<T> &r_v_vec) const
	{
		return Vector2<T>(x - r_v_vec.x, y - r_v_vec.y);
	}

	inline Vector2<T> operator -() const
	{
		return Vector2<T>(-x, -y);
	}

	inline Vector2<T> operator *(T t) const
	{
		return Vector2<T>(x * t, y * t);
	}

	inline Vector2<T> operator +=(const Vector2<T> &r_v_vec)
	{
		x += r_v_vec.x;
		y += r_v_vec.y;
		return *this;
	}

	inline Vector2<T> operator -=(const Vector2<T> &r_v_vec)
	{
		x -= r_v_vec.x;
		y -= r_v_vec.y;
		return *this;
	}

	inline Vector2<T> operator *=(T t)
	{
		x *= t;
		y *= t;
		return *this;
	}

	inline T f_Dot(const Vector2<T> &r_v_vec) const
	{
		return x * r_v_vec.x + y * r_v_vec.y;
	}

	inline Vector2<T> v_Orthogonal() const
	{
		return Vector2<T>(-y, x);
	}

	swizzle2_out(x, y, v_xy)
	swizzle2_out(y, x, v_yx)
};

typedef Vector2<float> Vector2f;
typedef Vector2<int> Vector2i;

/*
 *								=== ~Vector2 ===
 */

/*
 *								=== Vector3 ===
 */

template <class T> struct Vector3 {
	T x, y, z;
	//
	inline Vector3() {}
	inline Vector3(T t_x, T t_y, T t_z) :x(t_x), y(t_y), z(t_z) { }
	//
	inline T f_Length() const { return (T)sqrt(x * x + y * y + z * z); }
	inline T f_Length2() const { return x * x + y * y + z * z; }

	inline T operator [](int n_index) const
	{
		if(&x + 1 == &y && &x + 2 == &z) // should optimize away in release mode
			return *(&x + n_index);
		else {
			if(n_index == 0)
				return x;
			else if(n_index == 1)
				return y;
			else
				return z;
		}
	}

	inline T &operator [](int n_index)
	{
		if(&x + 1 == &y && &x + 2 == &z) // should optimize away in release mode
			return *(&x + n_index);
		else {
			if(n_index == 0)
				return x;
			else if(n_index == 1)
				return y;
			else
				return z;
		}
	}

	inline void Normalize()
	{
		T t = f_Length();
		
		if(t != 0) {
			t = 1 / t;
			x *= t;
			y *= t;
			z *= t;
		}
	}

	inline void Normalize(T t_len)
	{
		T t = f_Length();

		if(t != 0) {
			t = t_len / t;
			x *= t;
			y *= t;
			z *= t;
		}
	}

	inline bool operator ==(const Vector3<T> &r_v_vec) const
	{
		return x == r_v_vec.x && y == r_v_vec.y && z == r_v_vec.z;
	}

	inline Vector3<T> operator +(const Vector3<T> &r_v_vec) const
	{
		return Vector3<T>(x + r_v_vec.x, y + r_v_vec.y, z + r_v_vec.z);
	}

	inline Vector3<T> operator -(const Vector3<T> &r_v_vec) const
	{
		return Vector3<T>(x - r_v_vec.x, y - r_v_vec.y, z - r_v_vec.z);
	}

	inline Vector3<T> operator -() const
	{
		return Vector3<T>(-x, -y, -z);
	}

	inline Vector3<T> operator *(T t) const
	{
		return Vector3<T>(x * t, y * t, z * t);
	}

	inline Vector3<T> operator +=(const Vector3<T> &r_v_vec)
	{
		x += r_v_vec.x;
		y += r_v_vec.y;
		z += r_v_vec.z;
		return *this;
	}

	inline Vector3<T> operator -=(const Vector3<T> &r_v_vec)
	{
		x -= r_v_vec.x;
		y -= r_v_vec.y;
		z -= r_v_vec.z;
		return *this;
	}

	inline Vector3<T> operator *=(T t)
	{
		x *= t;
		y *= t;
		z *= t;
		return *this;
	}

	inline T f_Dot(const Vector3<T> &r_v_vec) const // dot product
	{
		return x * r_v_vec.x + y * r_v_vec.y + z * r_v_vec.z;
	}

	inline Vector3<T> v_Cross(const Vector3<T> &r_v_vec) const // cross product
	{
		return Vector3<T>(r_v_vec.y * z - r_v_vec.z * y, r_v_vec.z * x - r_v_vec.x * z, r_v_vec.x * y - r_v_vec.y * x);
	}

	inline Vector3<T> Cross(const Vector3<T> &r_v_vec) // cross product
	{
		T tx, ty;

		tx = r_v_vec.y * z - r_v_vec.z * y;
		ty = r_v_vec.z * x - r_v_vec.x * z;
		z = r_v_vec.x * y - r_v_vec.y * x;
		x = tx;
		y = ty;
		return *this;
	}

	float f_TE(const Vector3<T> &r_v_normal, const Vector3<T> &r_v_transmission,
		float f_n1_to_mi1, float f_n2_to_mi2) const
	{
		/*
		Fresnel law for transmission of light,
		polarized prependicular to surface of a dielectric

		n1, n2 are dielectric constants
		mi1, mi2 are magnetic permeabilities

		t_|_ = (2 * n1 / mi1 * cos(psi_i)) / (n1 / mi1 * cos(psi_i) + n2 / mi2 * cos(psi_t))
		*/

		float f_cos_psi_i = f_Dot(r_v_normal);
		float f_cos_psi_t = -r_v_normal.f_Dot(r_v_transmission);

		return 2 * f_n1_to_mi1 * f_cos_psi_i /
			(f_n1_to_mi1 * f_cos_psi_i + f_n2_to_mi2 * f_cos_psi_t);
	}

	float f_TM(const Vector3<T> &r_v_normal, const Vector3<T> &r_v_transmission,
		float f_n1_to_mi1, float f_n2_to_mi2) const
	{
		/*
		Fresnel law for transmission of light,
		polarized parallel to surface of a dielectric

		n1, n2 are dielectric constants
		mi1, mi2 are magnetic permeabilities

		t|| = (2 * n1 / mi1 * cos(psi_i)) / (n1 / mi1 * cos(psi_t) + n2 / mi2 * cos(psi_i))
		*/

		float f_cos_psi_i = f_Dot(r_v_normal);
		float f_cos_psi_t = -r_v_normal.f_Dot(r_v_transmission);

		return 2 * f_n1_to_mi1 * f_cos_psi_i /
			(f_n1_to_mi1 * f_cos_psi_t + f_n2_to_mi2 * f_cos_psi_i);
	}

	/*
	 *	float f_Fresnel_T(const Vector3<T> &r_v_normal, const Vector3<T> &r_v_transmission,
	 *		float f_n1_to_mi1, float f_n2_to_mi2)
	 *		- calculates transmission for ray that goes trough surface with normal r_v_normal
	 *		  and refracts in direction r_v_transmission (reflectance is 1 - transmission)
	 *		- f_n1_to_mi1 = n1 / mi1 = dielectric constant / magnetic permeability
	 *		- todo - find out if it actually works, look for some common constants
	 */
	float f_Fresnel_T(const Vector3<T> &r_v_normal, const Vector3<T> &r_v_transmission,
		float f_n1_to_mi1, float f_n2_to_mi2) const
	{
		// Fresnel law for transmission of unpolarized light t = .5 * (f_TM^2 + f_TE^2)

		float f_cos_psi_i = f_Dot(r_v_normal);
		float f_cos_psi_t = -r_v_normal.f_Dot(r_v_transmission);

		float f_te = 2 * f_n1_to_mi1 * f_cos_psi_i /
			(f_n1_to_mi1 * f_cos_psi_i + f_n2_to_mi2 * f_cos_psi_t);
		float f_tm = 2 * f_n1_to_mi1 * f_cos_psi_i /
			(f_n1_to_mi1 * f_cos_psi_t + f_n2_to_mi2 * f_cos_psi_i);

		return .5f * (f_te * f_te + f_tm * f_tm);
	}

	/*
	 *	inline Vector3<T> v_Refraction(const Vector3<T> &r_v_normal, float f_eta)
	 *		- calcualte refraction of vector coming trough surface with normal r_v_normal
	 *		- f_eta = index of refraction of first enviroment to index of refraction of second
	 *		- n(vacuum) = 1, n(air) = 1.00029, n(water) = 1.33,
	 *		  n(glass) = 1.52 - 1.62, n(diamond) = 2.417, n(plastic) = 1.46 - 1.55
	 */
	inline Vector3<T> v_Refraction(const Vector3<T> &r_v_normal, float f_eta) const
	{
		Vector3<T> v_tangent = v_Orthogonal(r_v_normal);
		float f_sin2_psi2 = f_eta * f_eta * (1.0f -
			r_v_normal.f_Dot(*this) * r_v_normal.f_Dot(*this));
		return (r_v_normal * -(float)sqrt(1.0f - f_sin2_psi2)) +
			   (v_tangent * (float)sqrt(f_sin2_psi2));
		// v_transmitted = cos(psi2) * (-r_v_normal) + sin(psi2) * v_tangent
	}

	inline Vector3<T> Refract(const Vector3<T> &r_v_normal, float f_eta)
	{
		return (*this = v_Refraction(r_v_normal, f_eta));
	}

	inline Vector3<T> v_Reflection(const Vector3<T> &r_v_normal) const
	{
		return (*this) - ((r_v_normal * f_Dot(r_v_normal)) * 2.0f);
	}

	inline Vector3<T> Reflect(const Vector3<T> &r_v_normal)
	{
		return ((*this) -= ((r_v_normal * f_Dot(r_v_normal)) * 2.0f));
	}

	inline Vector3<T> v_Orthogonal(const Vector3<T> &r_v_normal) const
	{
		Vector3<T> v_tangent;
		v_tangent = *this - r_v_normal * f_Dot(r_v_normal);
		v_tangent.Normalize();
		return v_tangent;
	}

	inline Vector3<T> Orthogonalize(const Vector3<T> &r_v_normal)
	{
		*this -= r_v_normal * f_Dot(r_v_normal);
		Normalize();
		return *this;
	}

	swizzle2_out(x, y, v_xy)
	swizzle2_out(x, z, v_xz)
	swizzle2_out(y, x, v_yx)
	swizzle2_out(y, z, v_yz)
	swizzle2_out(z, x, v_zx)
	swizzle2_out(z, y, v_zy)

	swizzle2_in(x, y, xy)
	swizzle2_in(x, z, xz)
	swizzle2_in(y, z, yz)

	swizzle3_out(x, y, z, v_xyz)
	swizzle3_out(x, z, y, v_xzy)
	swizzle3_out(y, x, z, v_yxz)
	swizzle3_out(y, z, x, v_yzx)
	swizzle3_out(z, x, y, v_zxy)
	swizzle3_out(z, y, x, v_zyx)
};

typedef Vector3<float> Vector3f;
typedef Vector3<int> Vector3i;

/*
 *								=== ~Vector3 ===
 */

/*
 *								=== Vector4 ===
 */

template <class T> struct Vector4 {
	T x, y, z, w;

	inline Vector4() {}
	inline Vector4(T t_x, T t_y, T t_z, T t_w) :x(t_x), y(t_y), z(t_z), w(t_w) { }

	inline T f_Length() const { return (T)sqrt(x * x + y * y + z * z + w * w); }
	inline T f_Length2() const { return x * x + y * y + z * z + w * w; }

	inline T operator [](int n_index) const
	{
		if(&x + 1 == &y && &x + 2 == &z && &x + 3 == &w) // should optimize away in release mode
			return *(&x + n_index);
		else {
			if(n_index == 0)
				return x;
			else if(n_index == 1)
				return y;
			else if(n_index == 2)
				return z;
			else
				return w;
		}
	}

	inline T &operator [](int n_index)
	{
		if(&x + 1 == &y && &x + 2 == &z && &x + 3 == &w) // should optimize away in release mode
			return *(&x + n_index);
		else {
			if(n_index == 0)
				return x;
			else if(n_index == 1)
				return y;
			else if(n_index == 2)
				return z;
			else
				return w;
		}
	}

	inline void Normalize()
	{
		T t = f_Length();
		
		if(t != 0) {
			t = 1 / t;
			x *= t;
			y *= t;
			z *= t;
			w *= t;
		}
	}

	inline void Normalize(T t_len)
	{
		T t = f_Length();

		if(t != 0) {
			t = t_len / t;
			x *= t;
			y *= t;
			z *= t;
			y *= t;
		}
	}

	inline bool operator ==(const Vector4<T> &r_v_vec) const
	{
		return x == r_v_vec.x && y == r_v_vec.y && z == r_v_vec.z && w == r_v_vec.w;
	}

	inline Vector4<T> operator +(const Vector4<T> &r_v_vec) const
	{
		return Vector4<T>(x + r_v_vec.x, y + r_v_vec.y, z + r_v_vec.z, w + r_v_vec.w);
	}

	inline Vector4<T> operator -(const Vector4<T> &r_v_vec) const
	{
		return Vector4<T>(x - r_v_vec.x, y - r_v_vec.y, z - r_v_vec.z, w - r_v_vec.w);
	}

	inline Vector4<T> operator -() const
	{
		return Vector4<T>(-x, -y, -z, -w);
	}

	inline Vector4<T> operator *(T t) const
	{
		return Vector4<T>(x * t, y * t, z * t, w * t);
	}

	inline Vector4<T> operator +=(const Vector4<T> &r_v_vec)
	{
		x += r_v_vec.x;
		y += r_v_vec.y;
		z += r_v_vec.z;
		w += r_v_vec.w;
		return *this;
	}

	inline Vector4<T> operator -=(const Vector4<T> &r_v_vec)
	{
		x -= r_v_vec.x;
		y -= r_v_vec.y;
		z -= r_v_vec.z;
		w -= r_v_vec.w;
		return *this;
	}

	inline Vector4<T> operator *=(T t)
	{
		x *= t;
		y *= t;
		z *= t;
		w *= t;
		return *this;
	}

	inline T f_Dot(const Vector4<T> &r_v_vec) const
	{
		return x * r_v_vec.x + y * r_v_vec.y + z * r_v_vec.z + w * r_v_vec.w;
	}

	swizzle4_out(x, y, z, w, v_xyzw)
	swizzle4_out(x, y, w, z, v_xywz)
	swizzle4_out(x, z, y, w, v_xzyw)
	swizzle4_out(x, z, w, y, v_xzwy)
	swizzle4_out(x, w, y, z, v_xwyz)
	swizzle4_out(x, w, z, y, v_xwzy)
	swizzle4_out(y, x, z, w, v_yxzw)
	swizzle4_out(y, x, w, z, v_yxwz)
	swizzle4_out(y, z, x, w, v_yzxw)
	swizzle4_out(y, z, w, x, v_yzwx)
	swizzle4_out(y, w, x, z, v_ywxz)
	swizzle4_out(y, w, z, x, v_ywzx)
	swizzle4_out(z, x, y, w, v_zxyw)
	swizzle4_out(z, x, w, y, v_zxwy)
	swizzle4_out(z, y, x, w, v_zyxw)
	swizzle4_out(z, y, w, x, v_zywx)
	swizzle4_out(z, w, x, y, v_zwxy)
	swizzle4_out(z, w, y, x, v_zwyx)
	swizzle4_out(w, x, y, z, v_wxyz)
	swizzle4_out(w, x, z, y, v_wxzy)
	swizzle4_out(w, y, x, z, v_wyxz)
	swizzle4_out(w, y, z, x, v_wyzx)
	swizzle4_out(w, z, x, y, v_wzxy)
	swizzle4_out(w, z, y, x, v_wzyx)

	swizzle3_out(x, y, z, v_xyz)
	swizzle3_out(x, y, w, v_xyw)
	swizzle3_out(x, z, y, v_xzy)
	swizzle3_out(x, z, w, v_xzw)
	swizzle3_out(x, w, y, v_xwy)
	swizzle3_out(x, w, z, v_xwz)
	swizzle3_out(y, x, z, v_yxz)
	swizzle3_out(y, x, w, v_yxw)
	swizzle3_out(y, z, x, v_yzx)
	swizzle3_out(y, z, w, v_yzw)
	swizzle3_out(y, w, x, v_ywx)
	swizzle3_out(y, w, z, v_ywz)
	swizzle3_out(z, x, y, v_zxy)
	swizzle3_out(z, x, w, v_zxw)
	swizzle3_out(z, y, x, v_zyx)
	swizzle3_out(z, y, w, v_zyw)
	swizzle3_out(z, w, x, v_zwx)
	swizzle3_out(z, w, y, v_zwy)
	swizzle3_out(w, x, y, v_wxy)
	swizzle3_out(w, x, z, v_wxz)
	swizzle3_out(w, y, x, v_wyx)
	swizzle3_out(w, y, z, v_wyz)
	swizzle3_out(w, z, x, v_wzx)
	swizzle3_out(w, z, y, v_wzy)

	swizzle2_out(x, y, v_xy)
	swizzle2_out(x, z, v_xz)
	swizzle2_out(x, w, v_xw)
	swizzle2_out(y, x, v_yx)
	swizzle2_out(y, z, v_yz)
	swizzle2_out(y, w, v_yw)
	swizzle2_out(z, x, v_zx)
	swizzle2_out(z, y, v_zy)
	swizzle2_out(z, w, v_zw)
	swizzle2_out(w, x, v_wx)
	swizzle2_out(w, y, v_wy)
	swizzle2_out(w, z, v_wz)

	swizzle2_in(x, y, xy)
	swizzle2_in(x, z, xz)
	swizzle2_in(x, w, xw)
	swizzle2_in(y, z, yz)
	swizzle2_in(y, w, yw)
	swizzle2_in(z, w, zw)

	swizzle3_in(x, y, z, xyz)
	swizzle3_in(x, y, w, xyw)
	swizzle3_in(x, z, w, xzw)
	swizzle3_in(y, z, w, yzw)
};

typedef Vector4<int> Vector4i;
typedef Vector4<float> Vector4f;
typedef Vector4<float> Color4f;

/*
 *								=== Vector4 ===
 */

/*
 *								=== Plane3f ===
 */

enum EPlanePos {
	plane_Front,
	plane_Back,
	plane_Onplane,
	plane_Split
};

struct Plane3f {
	Vector3f v_normal;
	float f_dist;

	inline Plane3f() {}
	Plane3f(const Vector3f &r_v_pos, const Vector3f &r_v_norm);
	Plane3f(const Vector3f &r_v_u, const Vector3f &r_v_v, const Vector3f &r_v_pos);

	EPlanePos n_Tri_Pos(const Vector3f &r_v_a,
		const Vector3f &r_v_b, const Vector3f &r_v_c, float f_epsilon_ex = f_epsilon) const;
	EPlanePos n_Vector_Pos(const Vector3f &r_v_vec, float f_epsilon_ex = f_epsilon) const;
	float f_Vector_Dist(const Vector3f &r_v_vec) const;
	float f_Vector_Dot(const Vector3f &r_v_vec) const;

	class CRayIsParallel_Exception {}; // thrown by v_Intersect_Ray, f_Intersect_Ray_t

	Vector3f v_Intersect_Ray(const Vector3f &r_v_org, const Vector3f &r_v_dir) const; // throw CRayIsParallel_Exception
	float f_Intersect_Ray_t(const Vector3f &r_v_org, const Vector3f &r_v_dir) const; // throw CRayIsParallel_Exception

	bool operator ==(const Plane3f &r_t_plane) const;
};

/*
 *								=== ~Plane3f ===
 */

/*
 *								=== Matrix4f ===
 */

struct Matrix4f {
protected:
	float f[4][4];

public:
	void Identity();
	void Translate(const Vector3f &r_v_dir);
	void Scale(float f_scale);
	void Scale(const Vector3f &r_v_scale);
	void RotateX(float f_angle);
	void RotateY(float f_angle);
	void RotateZ(float f_angle);
	void Rotate(float f_angle, const Vector3f &r_v_axis);

	Matrix4f operator *(const Matrix4f &r_m_mat) const;
	Matrix4f operator *=(const Matrix4f &r_m_mat);

	inline float *operator [](int n_index) { return f[n_index]; }
	inline const float *operator [](int n_index) const { return f[n_index]; }

	void Invert();
	Matrix4f t_Inverse() const;
	Matrix4f t_Transpose() const;

	Vector3f v_Transform_Pos(const Vector3f &r_v_vec) const;
	Vector3f v_Transform_Dir(const Vector3f &r_v_vec) const;
	Vector4f v_Transform(const Vector4f &r_v_vec) const;

	inline Vector3f v_Right() const { return Vector3f(f[0][0], f[0][1], f[0][2]); }
	inline Vector3f v_Up() const { return Vector3f(f[1][0], f[1][1], f[1][2]); }
	inline Vector3f v_Dir() const { return Vector3f(f[2][0], f[2][1], f[2][2]); }
	inline Vector3f v_Offset() const { return Vector3f(f[3][0], f[3][1], f[3][2]); }

	inline void Right(const Vector3f &r_v_vec)
	{
		f[0][0] = r_v_vec.x;
		f[0][1] = r_v_vec.y;
		f[0][2] = r_v_vec.z;
	}

	inline void Up(const Vector3f &r_v_vec)
	{
		f[1][0] = r_v_vec.x;
		f[1][1] = r_v_vec.y;
		f[1][2] = r_v_vec.z;
	}

	inline void Dir(const Vector3f &r_v_vec)
	{
		f[2][0] = r_v_vec.x;
		f[2][1] = r_v_vec.y;
		f[2][2] = r_v_vec.z;
	}

	inline void Offset(const Vector3f &r_v_vec)
	{
		f[3][0] = r_v_vec.x;
		f[3][1] = r_v_vec.y;
		f[3][2] = r_v_vec.z;
	}
};

/*
 *								=== ~Matrix4f ===
 */

/*
 *								=== CVectorMath ===
 */

class CVectorMath {
public:
	/*
	 *	static Vector3f v_Project(const Vector3f &r_v_vec, const Vector3f &r_v_ray_org,
	 *		const Vector3f &r_v_ray_dir, float f_ray_dir_length2 = 1.0f)
	 *		- project vector r_v_vec onto ray coming trough point r_v_ray_org
	 *		  in direction r_v_ray_dir
	 *		- f_ray_dir_length2 is squared length of r_v_ray_dir (or 1 in case it's unit length)
	 */
	static Vector3f v_Project(const Vector3f &r_v_vec, const Vector3f &r_v_ray_org,
		const Vector3f &r_v_ray_dir, float f_ray_dir_length2 = 1.0f)
	{
		return r_v_ray_org + r_v_ray_dir *
			(r_v_ray_dir.f_Dot(r_v_vec - r_v_ray_org) / f_ray_dir_length2);
	}

	/*
	 *	static Vector3f v_Nearest_Point(const Vector3f &r_v_ray_org, const Vector3f &r_v_ray_dir,
	 *		const Vector3f &r_v_ray2_org, const Vector3f &r_v_ray2_dir)
	 *		- returns nearest point, lying on ray [r_v_ray_org, r_v_ray_dir] to ray
	 *		  [r_v_ray2_org, r_v_ray2_dir]
	 *		- ray directions don't have to be unit length
	 *		- doesn't work for colinear rays - every point on ray is nearest in such a case
	 */
	static Vector3f v_Nearest_Point(const Vector3f &r_v_ray_org, const Vector3f &r_v_ray_dir,
		const Vector3f &r_v_ray2_org, const Vector3f &r_v_ray2_dir)
	{
		Plane3f t_coplane(r_v_ray2_org, r_v_ray2_dir.v_Cross(r_v_ray2_dir.v_Cross(r_v_ray_dir)));
		// plane, this line lies on and it' also prependicular to both this and r_t_line

		return t_coplane.v_Intersect_Ray(r_v_ray_org, r_v_ray_dir);
		// intersection of plane with this line is nearest point to r_t_line
		// (nearest points are connected by line, prependicular to both of them)
	}

	/*
	 * static inline float f_Det3(float f_00, float f_10, float f_20,
	 *							  float f_01, float f_11, float f_21,
	 *							  float f_02, float f_12, float f_22)
	 *		- calculates determinant of 3rd-order matrix
	 */
	static inline float f_Det3(float f_00, float f_10, float f_20,
							   float f_01, float f_11, float f_21,
							   float f_02, float f_12, float f_22)
	{
		return (f_00 * f_11 * f_22) + (f_10 * f_21 * f_02) + (f_01 * f_12 * f_20) -
			   (f_20 * f_11 * f_02) - (f_10 * f_01 * f_22) - (f_00 * f_21 * f_12);
	}

	/*
	 * static inline float f_Det2(float f_00, float f_10,
	 *							  float f_01, float f_11)
	 *		- calculates determinant of 2nd-order matrix
	 */
	static inline float f_Det2(float f_00, float f_10,
							   float f_01, float f_11)
	{
		return (f_00 * f_11) - (f_01 * f_10);
	}

	/*
	 *	static inline float f_Sqr(float f_x)
	 *		- calculates 2nd power of x
	 */
	static inline float f_Sqr(float f_x)
	{
		return f_x * f_x;
	}

	/*
	 *	static float f_Min_Distance(const Vector3f &r_v_ray_org, const Vector3f &r_v_ray_dir,
	 *		const Vector3f &r_v_ray2_org, const Vector3f &r_v_ray2_dir)
	 *		- returns minimal distance of ray [r_v_ray_org, r_v_ray_dir] to ray
	 *		  [r_v_ray2_org, r_v_ray2_dir]
	 *		- ray directions don't have to be unit length
	 *		- works for colinear rays as well
	 */
	static float f_Min_Distance(const Vector3f &r_v_ray_org, const Vector3f &r_v_ray_dir,
		const Vector3f &r_v_ray2_org, const Vector3f &r_v_ray2_dir)
	{
		Vector3f v_org_dist = r_v_ray_org - r_v_ray2_org;

		return (float)fabs(f_Det3(v_org_dist.x, v_org_dist.y, v_org_dist.z,
								  r_v_ray_dir.x, r_v_ray_dir.y, r_v_ray_dir.z,
								  r_v_ray2_dir.x, r_v_ray2_dir.y, r_v_ray2_dir.z)) / (float)sqrt(
			f_Sqr(f_Det2(r_v_ray_dir.y, r_v_ray_dir.z,
						 r_v_ray2_dir.y, r_v_ray2_dir.z)) +
			f_Sqr(f_Det2(r_v_ray_dir.z, r_v_ray_dir.x,
						 r_v_ray2_dir.z, r_v_ray2_dir.x)) +
			f_Sqr(f_Det2(r_v_ray_dir.x, r_v_ray_dir.y,
						 r_v_ray2_dir.x, r_v_ray2_dir.y)));
		// 1 sqrt, 21 mul, 1 div, 7 add, 3 sub = 8 + 42 + 2 + 7 + 3 = 62 ticks
		// formula from Hans-Jochen Bartsch's math book (results are almost identical
		// to those, obtained by meassuring distance between nerarest points on rays)
	}

	/*
	 *	struct TAxisAlignedBox
	 *		- bounding box, aligned with coordinate axes
	 */
	struct TAxisAlignedBox {
		Vector3f v_center; // box center
		Vector3f v_size; // box size along coordinate axes

		TAxisAlignedBox()
		{ }

		TAxisAlignedBox(const Vector3f &r_v_center, const Vector3f &r_v_size)
			:v_center(r_v_center), v_size(r_v_size)
		{ }

		/*
		 *	bool b_Intersect_Ray(const Vector3f &r_v_org, const Vector3f &r_v_dir) const
		 *		- returns true if ray [r_v_org, r_v_dir] intersects the box. otherwise false
		 */
		bool b_Intersect_Ray(const Vector3f &r_v_org, const Vector3f &r_v_dir) const
		{
			Vector3f v_diff = v_center - r_v_org;
			Vector3f v_ray_normal = r_v_dir * (r_v_dir.f_Dot(v_diff) / r_v_dir.f_Length());
			float f_effective_radius = Vector3f(v_ray_normal.x * v_size.x,
				v_ray_normal.y * v_size.y, v_ray_normal.z * v_size.z).f_Length();
			// should be divided by length of v_ray_normal, but ...

			return f_effective_radius <= v_ray_normal.f_Length2(); // we compensate here and save sqrt!
		}

		/*
		 *	bool b_Intersect_Segment(const Vector3f &r_v_org, const Vector3f &r_v_end) const
		 *		- returns true if segment [r_v_org, r_v_end] intersects the box. otherwise false
		 */
		bool b_Intersect_Segment(const Vector3f &r_v_org, const Vector3f &r_v_end) const
		{
			Vector3f v_half_dir = (r_v_end - r_v_org) * .5f;
			Vector3f v_seg_center = r_v_org + v_half_dir;
			Vector3f v_diff = v_center - v_seg_center;
			Vector3f v_dir_dot_axis;
			for(int i = 0; i < 3; ++ i) {
				v_dir_dot_axis[i] = (float)fabs(v_half_dir[i]);
				if((float)fabs(v_diff[i]) + v_size[i] > v_dir_dot_axis[i])
					return false;
			}
			// see if segment, projected to box axis miss the box

			Vector3f v_tangent = v_half_dir.v_Cross(v_diff);
			for(int i = 0; i < 3; ++ i) {
				const int n_index0[] = {1, 0, 0};
				const int n_index1[] = {2, 2, 1};
				if((float)fabs(v_tangent[i]) > v_size[n_index0[i]] * v_dir_dot_axis[n_index1[i]] +
				   v_size[n_index1[i]] * v_dir_dot_axis[n_index0[i]])
					return false;
			}
			// see for each axial projection if segment miss the box orthogonaly

			return true;
		}
	};

	/*
	 *	struct TObjectOrientedBox
	 *		- extension of TAxisAlignedBox, box is now aligned to some arbitrary axes
	 *		  (but they still ought to form right-handed euclidean coordinate system)
	 */
	struct TObjectOrientedBox : public TAxisAlignedBox {
		Vector3f v_axis[3]; // normalized axes, box size is along those axes

		TObjectOrientedBox()
		{ }

		TObjectOrientedBox(const Vector3f &r_v_center, const Vector3f &r_v_size,
			const Vector3f _v_axis[3])
			:TAxisAlignedBox(r_v_center, r_v_size)
		{
			memcpy(v_axis, _v_axis, 3 * sizeof(Vector3f));
			for(int i = 0; i < 3; ++ i)
				v_axis[i].Normalize();
		}

		TObjectOrientedBox(const Vector3f &r_v_center, const Vector3f &r_v_size,
			const Matrix4f &r_t_matrix)
			:TAxisAlignedBox(r_v_center, r_v_size)
		{
			v_axis[0] = r_t_matrix.v_Right();
			v_axis[1] = r_t_matrix.v_Up();
			v_axis[2] = r_t_matrix.v_Dir();
			for(int i = 0; i < 3; ++ i)
				v_axis[i].Normalize();
		}

		TObjectOrientedBox(const TAxisAlignedBox &r_t_box)
			:TAxisAlignedBox(r_t_box.v_center, r_t_box.v_size)
		{
			v_axis[0] = Vector3f(1, 0, 0);
			v_axis[1] = Vector3f(0, 1, 0);
			v_axis[2] = Vector3f(0, 0, 1);
		}

		/*
		 *	bool b_Intersect_Ray(const Vector3f &r_v_org, const Vector3f &r_v_dir) const
		 *		- returns true if ray [r_v_org, r_v_dir] intersects the box. otherwise false
		 */
		bool b_Intersect_Ray(const Vector3f &r_v_org, const Vector3f &r_v_dir) const
		{
			Vector3f v_diff = v_center - r_v_org;
			Vector3f v_ray_normal = r_v_dir * (r_v_dir.f_Dot(v_diff) / r_v_dir.f_Length());
			float f_effective_radius = Vector3f(v_axis[0].f_Dot(v_ray_normal) * v_size.x,
				v_axis[2].f_Dot(v_ray_normal) * v_size.y,
				v_axis[3].f_Dot(v_ray_normal) * v_size.z).f_Length();
			// should be divided by length of v_ray_normal, but ...

			return f_effective_radius <= v_ray_normal.f_Length2(); // we compensate here and save sqrt!
		}

		/*
		 *	bool b_Intersect_Segment(const Vector3f &r_v_org, const Vector3f &r_v_end) const
		 *		- returns true if segment [r_v_org, r_v_end] intersects the box. otherwise false
		 */
		bool b_Intersect_Segment(const Vector3f &r_v_org, const Vector3f &r_v_end) const
		{
			Vector3f v_half_dir = (r_v_end - r_v_org) * .5f;
			Vector3f v_seg_center = r_v_org + v_half_dir;
			Vector3f v_diff = v_center - v_seg_center;
			Vector3f v_dir_dot_axis;
			for(int i = 0; i < 3; ++ i) {
				v_dir_dot_axis[i] = (float)fabs(v_half_dir.f_Dot(v_axis[i]));
				if((float)fabs(v_diff.f_Dot(v_axis[i])) + v_size[i] > v_dir_dot_axis[i])
					return false;
			}
			// see if segment, projected to box axis miss the box

			Vector3f v_tangent = v_half_dir.v_Cross(v_diff);
			for(int i = 0; i < 3; ++ i) {
				const int n_index0[] = {1, 0, 0};
				const int n_index1[] = {2, 2, 1};
				if((float)fabs(v_tangent.f_Dot(v_axis[i])) >
				   v_size[n_index0[i]] * v_dir_dot_axis[n_index1[i]] +
				   v_size[n_index1[i]] * v_dir_dot_axis[n_index0[i]])
					return false;
			}
			// see for each axial projection if segment miss the box orthogonaly

			return true;
		}
	};
};

/*
 *								=== ~CVectorMath ===
 */

#endif // __VECTOR2_INCLUDED
